#include <gtk/gtk.h>
#include "xpiezo.h"
#include "xpiezo-gui.h"

#define _(arg) arg


void
xpiezo_load_pixmaps (XPiezoContext *context)
{
  GdkPixbuf *ufo0, *ufo45, *arrow, *tmp1;
  gint i;

  g_return_if_fail (context->ufo_pixbufs == NULL);

  context->ufo_pixbufs = g_new (GdkPixbuf *, 8);
  ufo0 = gdk_pixbuf_new_from_file ("ufo-icon-00.png", NULL);
  ufo45 = gdk_pixbuf_new_from_file ("ufo-icon-45.png", NULL);
  arrow = gdk_pixbuf_new_from_file ("arrow.png", NULL);

  for (i = 0; i < 8; i++)
    {
      tmp1 = gdk_pixbuf_rotate_simple ((i % 2) == 0 ? ufo0 : ufo45,
                                       (360 - (i / 2) * 90) % 360);
      gdk_pixbuf_composite (arrow, tmp1, 0, 0, 80, 80,
                            (i < 4) ? ((i > 0) ? -24 : 24) : 24, 0,
                            1.0, 1.0, GDK_INTERP_NEAREST, 255);
      context->ufo_pixbufs[i] = tmp1;
    }

  g_object_unref (ufo0);
  g_object_unref (ufo45);
  g_object_unref (arrow);
}


static void
xpiezo_orientation_callback (GtkRadioButton *radio,
                             gpointer        user_data)
{
  XPiezoContext *context = (XPiezoContext *) user_data;
  gint dir;

  if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio)))
    return;

  dir = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (radio), "direction"));

  gtk_image_set_from_pixbuf (GTK_IMAGE (context->ufo_image),
                             context->ufo_pixbufs[dir]);
}


static void
xpiezo_create_toolbar (XPiezoContext *context,
                       GtkWidget     *parent)
{
  GtkWidget *toolbar, *button, *image, *frame, *statusbar;

  toolbar = g_object_new (GTK_TYPE_TOOLBAR,
                          "visible", TRUE,
                          NULL);
  gtk_box_pack_start (GTK_BOX (parent), toolbar, FALSE, FALSE, 0);

  button = g_object_new (GTK_TYPE_TOOL_BUTTON,
                         "stock-id", "gtk-connect",
                         "parent", toolbar,
                         "visible", TRUE,
                         NULL);

  button = g_object_new (GTK_TYPE_TOOL_BUTTON,
                         "stock-id", "gtk-disconnect",
                         "parent", toolbar,
                         "visible", TRUE,
                         NULL);

  button = g_object_new (GTK_TYPE_TOOL_BUTTON,
                         "stock-id", "gtk-revert-to-saved",
                         "parent", toolbar,
                         "visible", TRUE,
                         NULL);

  button = g_object_new (GTK_TYPE_TOOL_BUTTON,
                         "stock-id", "gtk-save",
                         "parent", toolbar,
                         "visible", TRUE,
                         NULL);

  statusbar = g_object_new (GTK_TYPE_STATUSBAR,
                            "homogeneous", FALSE,
                            "visible", TRUE,
                            NULL);
  gtk_box_pack_end (GTK_BOX (parent), statusbar, FALSE, FALSE, 0);

  frame = g_object_new (GTK_TYPE_FRAME,
                        "shadow", GTK_SHADOW_IN,
                        "visible", TRUE,
                        NULL);
  image = g_object_new (GTK_TYPE_IMAGE,
                        "stock", "gtk-dialog-warning",
                        "xpad", 1,
                        "ypad", 1,
                        "parent", frame,
                        "visible", TRUE,
                        NULL);
  gtk_box_pack_start (GTK_BOX (statusbar), frame, FALSE, FALSE, 0);
  gtk_box_reorder_child (GTK_BOX (statusbar), frame, 0);

  frame = g_object_new (GTK_TYPE_FRAME,
                        "shadow", GTK_SHADOW_IN,
                        "visible", TRUE,
                        NULL);
  image = g_object_new (GTK_TYPE_IMAGE,
                        "stock", "gtk-connect",
                        "xpad", 1,
                        "ypad", 1,
                        "parent", frame,
                        "visible", TRUE,
                        NULL);
  gtk_box_pack_start (GTK_BOX (statusbar), frame, FALSE, FALSE, 0);
  gtk_box_reorder_child (GTK_BOX (statusbar), frame, 0);
}

static void
xpiezo_create_orientation (XPiezoContext *context,
                           GtkWidget     *parent)
{
  GtkWidget *label, *frame, *alignment, *table, *radio, *image;
  GSList *rgroup;
  gint rcoords[] = { 2,0, 3,1, 4,2, 3,3, 2,4, 1,3, 0,2, 1,1 };
  gint i;

  label = g_object_new (GTK_TYPE_LABEL,
                        "label", _("<b>Orientation</b>"),
                        "use-markup", TRUE,
                        "visible", TRUE,
                        NULL);


  frame = g_object_new (GTK_TYPE_FRAME,
                        "label-widget", label,
                        "shadow-type", GTK_SHADOW_NONE,
                        "visible", TRUE,
                        NULL);
  gtk_box_pack_start (GTK_BOX (parent), frame, FALSE, FALSE, 0);

  alignment = g_object_new (GTK_TYPE_ALIGNMENT,
                            "left-padding", 12,
                            "parent", frame,
                            "visible", TRUE,
                            NULL);

  table = g_object_new (GTK_TYPE_TABLE,
                        "n-rows", 5,
                        "n-columns", 5,
                        "parent", alignment,
                        "visible", TRUE,
                        NULL);

  rgroup = NULL;

  for (i = 0; i < 8; i++)
    {
      radio = gtk_radio_button_new (rgroup);
      gtk_widget_show (radio);

      g_object_set_data (G_OBJECT (radio), "direction", GINT_TO_POINTER (i));
      g_signal_connect (radio, "toggled",
                        G_CALLBACK (xpiezo_orientation_callback),
                        context);

      gtk_table_attach (GTK_TABLE (table), radio,
                        rcoords [i*2], rcoords[i*2] + 1,
                        rcoords [i*2+1], rcoords[i*2+1] + 1,
                        0, 0, 0, 0);
      rgroup = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio));
    }

  image = g_object_new (GTK_TYPE_IMAGE,
                        "pixbuf", context->ufo_pixbufs[0],
                        "visible", TRUE,
                        "height-request", 80,
                        "width-request", 80,
                        NULL);

  context->ufo_image = image;

  gtk_table_attach (GTK_TABLE (table), image, 2, 3, 2, 3, 0, 0, 0, 0);
}

static void
xpiezo_create_easy_params (XPiezoContext *context,
                           GtkWidget     *parent)
{
  GtkWidget *table, *hbox, *label, *scale, *button;
  GtkObject *adjustment;

  table = g_object_new (GTK_TYPE_TABLE,
                        "n-rows", 5,
                        "n-columns", 4,
                        "parent", parent,
                        "visible", TRUE,
                        NULL);

  hbox = g_object_new (GTK_TYPE_HBOX,
                       "spacing", 12,
                       "homogeneous", TRUE,
                       "visible", TRUE,
                       NULL);

  label = g_object_new (GTK_TYPE_LABEL,
                        "label", _("<i><small>lightweight</small></i>"),
                        "use-markup", TRUE,
                        "xalign", 0.0,
                        "justify", GTK_JUSTIFY_LEFT,
                        "parent", hbox,
                        "visible", TRUE,
                        NULL);
  label = g_object_new (GTK_TYPE_LABEL,
                        "label", _("<i><small>medium</small></i>"),
                        "use-markup", TRUE,
                        "xalign", 0.5,
                        "justify", GTK_JUSTIFY_CENTER,
                        "parent", hbox,
                        "visible", TRUE,
                        NULL);
  label = g_object_new (GTK_TYPE_LABEL,
                        "label", _("<i><small>heavy</small></i>"),
                        "use-markup", TRUE,
                        "xalign", 1.0,
                        "justify", GTK_JUSTIFY_RIGHT,
                        "parent", hbox,
                        "visible", TRUE,
                        NULL);
  gtk_table_attach (GTK_TABLE (table), hbox, 1, 4, 0, 1,
                    GTK_FILL, 0, 0, 0);

  label = g_object_new (GTK_TYPE_LABEL,
                        "label", _("Weight:"),
                        "xalign", 0.0,
                        "visible", TRUE,
                        NULL);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
                    GTK_FILL, 0, 0, 0);

  adjustment = gtk_adjustment_new (5, 0, 7, 1, 1, 0);
  scale = g_object_new (GTK_TYPE_HSCALE,
                        "digits", -1,
                        "draw-value", FALSE,
                        "adjustment", adjustment,
                        "visible", TRUE,
                        NULL);
  gtk_table_attach (GTK_TABLE (table), scale, 1, 4, 1, 2,
                    GTK_EXPAND | GTK_FILL, 0, 0, 0);

  label = g_object_new (GTK_TYPE_LABEL,
                        "label", _("Control:"),
                        "xalign", 0.0,
                        "visible", TRUE,
                        NULL);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
                    GTK_FILL, 0, 0, 0);

  adjustment = gtk_adjustment_new (2, 0, 7, 1, 1, 0);
  scale = g_object_new (GTK_TYPE_HSCALE,
                        "digits", -1,
                        "draw-value", FALSE,
                        "adjustment", adjustment,
                        "visible", TRUE,
                        NULL);
  gtk_table_attach (GTK_TABLE (table), scale, 1, 4, 2, 3,
                    GTK_EXPAND | GTK_FILL, 0, 0, 0);

  hbox = g_object_new (GTK_TYPE_HBOX,
                       "spacing", 12,
                       "homogeneous", TRUE,
                       "visible", TRUE,
                       NULL);

  label = g_object_new (GTK_TYPE_LABEL,
                        "label", _("<i><small>lazy, slow,\nindoor</small></i>"),
                        "use-markup", TRUE,
                        "xalign", 0.0,
                        "justify", GTK_JUSTIFY_LEFT,
                        "parent", hbox,
                        "visible", TRUE,
                        NULL);
  label = g_object_new (GTK_TYPE_LABEL,
                        "label", _("<i><small>agile, vivid,\noutdoor</small></i>"),
                        "use-markup", TRUE,
                        "xalign", 1.0,
                        "justify", GTK_JUSTIFY_RIGHT,
                        "parent", hbox,
                        "visible", TRUE,
                        NULL);
  gtk_table_attach (GTK_TABLE (table), hbox, 1, 4, 3, 4,
                    GTK_EXPAND | GTK_FILL, 0, 0, 0);

  label = g_object_new (GTK_TYPE_LABEL,
                        "label", _("Fast Yaw:"),
                        "xalign", 0.0,
                        "visible", TRUE,
                        NULL);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 4, 5,
                    GTK_FILL, 0, 0, 0);

  button = g_object_new (GTK_TYPE_CHECK_BUTTON,
                         "active", FALSE,
                         "visible", TRUE,
                         NULL);

  gtk_table_attach (GTK_TABLE (table), button, 1, 5, 4, 5,
                    GTK_FILL, 0, 0, 0);

}

static void
xpiezo_create_complex_params (XPiezoContext *context,
                              GtkWidget     *parent)
{
  GtkWidget *expander, *table, *label, *alignment, *spinbutton;
  GtkObject *adjustment;
  gint i;

  ParamPreset  nrpre[] = {
      { "K<sub>P</sub>:", 164, 0, 10000, 0 },
      { "K<sub>D</sub>:", 414, 0, 10000, 0 },
      { "angle_max:", 1000000, 0, 1000000, 0 },
      { "K_stick:", 10, 0, 100, 0 },
      { "K_expo:", 73, 0, 1000, 0 },
      { "K_expo_edge:", 3000, 0, 10000, 0 },
      { "K_stick_direct:", 11, 0, 1000, 0 },
  };

  ParamPreset  ypre[] = {
      { "K<sub>P</sub>:", 386, 0, 10000, 0 },
      { "K<sub>D</sub>:", 507, 0, 10000, 0 },
      { "K_stick:", 16, 0, 100, 0 },
      { "fast_yaw_margin:", 15, 0, 1000, 0 },
      { "K_fast_yaw:", 6939, 0, 10000, 0 },
      { "Temperature", 0, 0, 0, -5 },
      { "compensation:", 50, 0, 1000, 0 },

  };

  expander = g_object_new (GTK_TYPE_EXPANDER,
                           "label", "<b>Expert parameters</b>",
                           "use-markup", TRUE,
                           "expanded", FALSE,
                           "visible", TRUE,
                           NULL);
  gtk_box_pack_start (GTK_BOX (parent), expander, FALSE, FALSE, 0);

  alignment = g_object_new (GTK_TYPE_ALIGNMENT,
                            "left-padding", 12,
                            "parent", expander,
                            "visible", TRUE,
                            NULL);

  table = g_object_new (GTK_TYPE_TABLE,
                        "n-rows", 8,
                        "n-columns", 4,
                        "parent", alignment,
                        "visible", TRUE,
                        NULL);

  label = g_object_new (GTK_TYPE_LABEL,
                        "label", "Nick/Roll",
                        "visible", TRUE,
                        "xalign", 0.0,
                        NULL);
  gtk_table_attach (GTK_TABLE (table), label, 0, 2, 0, 1,
                    GTK_FILL, 0, 0, 0);

  for (i = 0; i < G_N_ELEMENTS (nrpre); i++)
    {
      if (nrpre[i].label == NULL)
        continue;

      label = g_object_new (GTK_TYPE_LABEL,
                            "label", nrpre[i].label,
                            "use-markup", TRUE,
                            "xpad", (nrpre[i].digits < -1) ? 0 : 12,
                            "visible", TRUE,
                            "xalign", 0.0,
                            NULL);
      gtk_table_attach (GTK_TABLE (table), label, 0, 1, i+1, i+2,
                        GTK_FILL, 0, 0, 0);

      if (nrpre[i].digits < -1)
        continue;

      adjustment = gtk_adjustment_new (nrpre[i].value,
                                       nrpre[i].min,
                                       nrpre[i].max,
                                       1, 1, 0);

      spinbutton = g_object_new (GTK_TYPE_SPIN_BUTTON,
                                 "adjustment", adjustment,
                                 "digits", nrpre[i].digits,
                                 "visible", TRUE,
                                 NULL);
      gtk_table_attach (GTK_TABLE (table), spinbutton, 1, 2, i+1, i+2,
                        GTK_FILL | GTK_EXPAND, 0, 0, 0);
    }

  label = g_object_new (GTK_TYPE_LABEL,
                        "label", "Yaw",
                        "visible", TRUE,
                        "xalign", 0.0,
                        NULL);
  gtk_table_attach (GTK_TABLE (table), label, 2, 4, 0, 1,
                    GTK_FILL, 0, 0, 0);

  for (i = 0; i < G_N_ELEMENTS (ypre); i++)
    {
      if (ypre[i].label == NULL)
        continue;

      label = g_object_new (GTK_TYPE_LABEL,
                            "label", ypre[i].label,
                            "use-markup", TRUE,
                            "xpad", (ypre[i].digits < -1) ? 0 : 12,
                            "visible", TRUE,
                            "xalign", 0.0,
                            NULL);
      gtk_table_attach (GTK_TABLE (table), label, 2, 3, i+1, i+2,
                        GTK_FILL, 0, 0, 0);

      if (ypre[i].digits < -1)
        continue;

      adjustment = gtk_adjustment_new (ypre[i].value,
                                       ypre[i].min,
                                       ypre[i].max,
                                       1, 1, 0);

      spinbutton = g_object_new (GTK_TYPE_SPIN_BUTTON,
                                 "adjustment", adjustment,
                                 "digits", ypre[i].digits,
                                 "visible", TRUE,
                                 NULL);
      gtk_table_attach (GTK_TABLE (table), spinbutton, 3, 4, i+1, i+2,
                        GTK_FILL | GTK_EXPAND, 0, 0, 0);
    }


}

GtkWidget *
xpiezo_create_window (XPiezoContext *context)
{
  GtkWidget *window, *main_vbox, *vbox,
            *label, *alignment, *frame;

  window = g_object_new (GTK_TYPE_WINDOW,
                         "type", GTK_WINDOW_TOPLEVEL,
                         "title", "xpiezo",
                         "default-height", 100,
                         "default-width", 100,
                         NULL);
  g_signal_connect (window, "delete_event", gtk_main_quit, NULL);

  main_vbox = g_object_new (GTK_TYPE_VBOX,
                            "parent", window,
                            "spacing", 10,
                            "visible", TRUE,
                            NULL);

  xpiezo_create_toolbar (context, main_vbox);
  xpiezo_create_orientation (context, main_vbox);

  /* Parameterarea: Easy parameters */

  label = g_object_new (GTK_TYPE_LABEL,
                        "label", _("<b>Parameters</b>"),
                        "use-markup", TRUE,
                        "visible", TRUE,
                        NULL);


  frame = g_object_new (GTK_TYPE_FRAME,
                        "label-widget", label,
                        "shadow-type", GTK_SHADOW_NONE,
                        "visible", TRUE,
                        NULL);
  gtk_box_pack_start (GTK_BOX (main_vbox), frame, FALSE, FALSE, 0);

  alignment = g_object_new (GTK_TYPE_ALIGNMENT,
                            "left-padding", 12,
                            "parent", frame,
                            "visible", TRUE,
                            NULL);

  vbox = g_object_new (GTK_TYPE_VBOX,
                       "parent", alignment,
                       "spacing", 10,
                       "visible", TRUE,
                       NULL);

  xpiezo_create_easy_params (context, vbox);

  xpiezo_create_complex_params (context, main_vbox);

  return window;
}

