/*
 * - Kp: P-Anteil Nick und Roll
 * - Kd: D-Anteil Nick und Roll
 * - angle_max: maximaler Winkel, der aufintegriert wird
 * - K-Stick: direkte lineare Stickwirkung auf den Winkel auf Nick und Roll
 * - Expo: (eigentlich x^2)-Anteil für Nick und Roll damit bei mehr stick
 *     weniger kontroller, dafür mehr speedunterschied auftritt
 * - K_expo_edge: ist die nicht-lineare Randverstärkung des Sticks für die
 *     Extrem-Turns (also ein Expo++ sozusagen)
 * - K_stick_direct: ist der Stick-Anteil für Nick und Roll der direkte am
 *     Winkel verbei wirkt ohne den ist das auf der Stelle schweben schwieriger
 * - Kp_yaw: P-Anteil Yaw
 * - Kd_Yaw: D-Anteil Yaw
 * - K_Stick_yaw: direkte lineare Stickwirkung auf den Winkel auf yaw
 * - fast yaw margin: gibt den Fast_Yaw_Mode in dem der Winkel igonieriert wird
 *     und der Margin bestimmt ab welcher Yaw-Speed der Winkel wieder aktiviert
 *     wird (0=erst bei Stillstand) solange wird nur der D abgebremst
 * - fast_yaw_enable macht das schaltet den Mode global ein und aus
 */

 /*
  * setting                     1         2         3         4
  *
  * kp                        220       178       178       180
  * kd                        470       428       428       470
  * angle_max             1000000   2000000   2000000   2000000
  * k_stick                     8        12        16        20
  * K_expo                     60        86       112       150
  * Kexpo_edge               1000      5000      9000     12000
  * K_stick_direkt             10        11        12        14
  *
  * kp_yaw                    410       392       392       400
  * kd_yaw                    535       514       514       540
  * K_stick_yaw                20        24        28        28
  * fast_yaw_margin            15        15        15        10
  * K_fast_yaw                  0        98       104       110
  * temp. comp.                60        50        50        50
  */

typedef enum _XPiezoOrientation XPiezoOrientation;
enum _XPiezoOrientation
{
  XPIEZO_ORI_000,
  XPIEZO_ORI_045,
  XPIEZO_ORI_090,
  XPIEZO_ORI_135,
  XPIEZO_ORI_180,
  XPIEZO_ORI_225,
  XPIEZO_ORI_270,
  XPIEZO_ORI_315,
};


typedef struct _XPiezoPreset XPiezoPreset;
struct _XPiezoPreset
{
  gboolean  easyparams;

  /* easy parameters. */
  XPiezoOrientation  orientation;
  gint               weight;   /* weight preset, lightweight to heavy, 0-7 */
  gint               agility;  /* lazy to agile, 0-7 */
  gboolean           fast_yaw;

  /* expert parameters */

  /* Nick/Roll */
  gint32   nr_kp;
  gint32   nr_kd;
  gint32   angle_max;
  gint32   nr_k_stick;
  gint32   nr_k_expo;
  gint32   nr_k_expo_edge;
  gint32   nr_k_stick_direct;

  /* Yaw */
  gint32   yaw_kp;
  gint32   yaw_kd;
  gint32   yaw_k_stick;
  gint32   fast_yaw_margin;
  gint32   k_fast_yaw;

  /* general */
  gint     temp_comp;
};


typedef struct _XPiezoConfig XPiezoConfig;
struct _XPiezoConfig
{
  XPiezoPreset  presets[4];
};


